import 'dart:async';

import 'package:flutter/services.dart';

class ZolozkitForFlutter {
  static const MethodChannel _channel =
      const MethodChannel('com.zoloz.flutter.plugin/zolozkit');
  static Function? _onCompleteCallback;
  static Function? _onInterruptedCallback;

  static Future<String?> get metaInfo async {
    final String? metaInfo = await _channel.invokeMethod('getMetaInfo');
    return metaInfo;
  }

  static Future<String?> get zolozLocale async {
    String? metaInfo = await _channel.invokeMethod("getLocaleKey");
    return metaInfo;
  }

  static Future<String?> get zolozChameleonConfigPath async {
    String? metaInfo = await _channel.invokeMethod("getChameleonConfigPath");
    return metaInfo;
  }

  static Future<String?> get zolozPublicKey async {
    String? metaInfo = await _channel.invokeMethod("getPublicKey");
    return metaInfo;
  }

  static Future<void> _methodCallHandler(MethodCall call) async {
    print("dart method callhandler");
    switch (call.method) {
      case 'onComplete':
        if (_onCompleteCallback != null) {
          _onCompleteCallback!(call.arguments["retCode"], call.arguments["extInfo"]);
          _onCompleteCallback = null;
        }
        _onInterruptedCallback = null;
        break;
      case 'onInterrupted':
        if (_onInterruptedCallback != null) {
          _onInterruptedCallback!(call.arguments["retCode"], call.arguments["extInfo"]);
          _onInterruptedCallback = null;
        }
        _onCompleteCallback = null;
        break;
    }
  }

  static Future start(String clientCfg, Map bizCfg, Function onInterrupted, Function onComplete) async {
    _channel.setMethodCallHandler(_methodCallHandler);
    _onCompleteCallback = onComplete;
    _onInterruptedCallback = onInterrupted;
    await _channel.invokeMethod("start",
        {"clientCfg": clientCfg, "bizCfg": bizCfg == null ? {} : bizCfg});
  }
}
